﻿local RC = LibStub:GetLibrary("LibResComm-1.0");
if not RC then return end

local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale("Grid2")

local Grid2StatusRes = Grid2:NewModule("StatusRes")

local ResIncoming = Grid2.statusPrototype:new("resurrection-incoming")
local Resurrected = Grid2.statusPrototype:new("resurrected")
local Soulstone = Grid2.statusPrototype:new("soulstone")

Grid2 = Grid2

--{{{ AceBucket-3.0, embedded upon use

function Grid2StatusRes:RegisterBucketEvent(...)
	LibStub("AceBucket-3.0"):Embed(Grid2StatusRes)
	return self:RegisterBucketEvent(...)
end

function Grid2StatusRes:RegisterBucketMessage(...)
	LibStub("AceBucket-3.0"):Embed(Grid2StatusRes)
	return self:RegisterBucketMessage(...)
end

--}}}

do
	local count = 0
	function Grid2StatusRes:EnableHealthFrame(enable)
		local prev = (count == 0)
		if enable then
			count = count + 1
		else
			count = count - 1
		end
		assert(count >= 0)
		local curr = (count == 0)
		if prev ~= curr then
			if curr then
				RC.UnregisterCallback(self, "ResComm_ResStart")
				RC.UnregisterCallback(self, "ResComm_ResEnd")
				RC.UnregisterCallback(self, "ResComm_Ressed")
				RC.UnRegisterCallback(self, "ResComm_CanRes")
				RC.UnRegisterCallback(self, "ResComm_ResExpired")
				self:UnregisterBucketEvent("UNIT_HEALTH")
			else
				RC.RegisterCallback(self, "ResComm_ResStart")
				RC.RegisterCallback(self, "ResComm_ResEnd")
				RC.RegisterCallback(self, "ResComm_Ressed")
				RC.RegisterCallback(self, "ResComm_CanRes")
				RC.RegisterCallback(self, "ResComm_ResExpired")
				self:RegisterBucketEvent("UNIT_HEALTH", 0.1, "HealthChanged")
			end
		end
	end
end



ResIncoming.defaultDB = {
	profile = {
		color1 = { r = 1, g = 0, b = 0, a = 1 },
	}
}
local unitResIncoming = {}

function ResIncoming:OnEnable()
	Grid2StatusRes:EnableHealthFrame(true)
end

function ResIncoming:OnDisable()
	Grid2StatusRes:EnableHealthFrame(false)
end

function ResIncoming:IsActive(unitid)
	return unitResIncoming[unitid]
end

function ResIncoming:GetColor(unit)
	local color = self.db.profile.color1
	return color.r, color.g, color.b, color.a
end

function ResIncoming:GetText(unit)
	return L["+RES"]
end

Grid2:RegisterStatus(ResIncoming, { "color", "text" })



Resurrected.defaultDB = {
	profile = {
		color1 = { r = 0, g = 1, b = 0, a = 1 },
	}
}
local unitResurrected = {}


Soulstone.defaultDB = {
	profile = {
		color1 = { r = 0, g = 1, b = 0, a = 1 },
	}
}
local unitSoulstone = {}

--[[
Grid2StatusRes.defaultDB = {
	debug = false,
	alert_res = {
		enable = true,
		color = { r = 1, g = 0, b = 0, a = 1 },
		priority = 50,
		range = false,
	},
	alert_ressed = {
		enable = true,
		color = { r = 0, g = 1, b = 0, a = 1 },
		priority = 51,
		range = false,
	},
	alert_ss = {
		enable = true,
		color = { r = 0, g = 1, b = 0, a = 1 },
		priority = 49,
		range = false,
	},
}
--]]

function Grid2StatusRes:ResComm_ResStart(event, _, _, targetName)
	local guid = Grid2:GetGUIDByFullName(targetName)
	local unitid = Grid2:GetUnitidByGUID(guid)

	self:Debug("ResComm_ResStart targetName", targetName, "unitid", unitid, "guid", guid)
	unitResIncoming[unitid] = true
	if (ResIncoming.enabled) then ResIncoming:UpdateIndicators(unitid) end
end

function Grid2StatusRes:ResComm_ResEnd(event, _, targetName)
	if (targetName and not RC:IsUnitBeingRessed(targetName)) then
		local guid = Grid2:GetGUIDByFullName(targetName)
		local unitid = Grid2:GetUnitidByGUID(guid)

		self:Debug("ResComm_ResEnd targetName", targetName, "unitid", unitid, "guid", guid)
		if (unitResIncoming[unitid]) then
			unitResIncoming[unitid] = nil
			if (ResIncoming.enabled) then ResIncoming:UpdateIndicators(unitid) end
		end
	end
end

function Grid2StatusRes:ResComm_ResExpired(event, ressed)
	local guid = Grid2:GetGUIDByFullName(ressed)
	local unitid = Grid2:GetUnitidByGUID(guid)

	self:Debug("ResComm_ResExpired ressed", ressed, "unitid", unitid, "guid", guid)
	if (unitResurrected[unitid]) then
		unitResurrected[unitid] = nil
		if (Resurrected.enabled) then Resurrected:UpdateIndicators(unitid) end
	end
end

function Grid2StatusRes:ResComm_Ressed(event, ressed)
	local guid = Grid2:GetGUIDByFullName(ressed)
	local unitid = Grid2:GetUnitidByGUID(guid)

	self:Debug("ResComm_Ressed ressed", ressed, "unitid", unitid, "guid", guid)
	unitResurrected[unitid] = "RES"
	if (Resurrected.enabled) then Resurrected:UpdateIndicators(unitid) end
end

function Grid2StatusRes:ResComm_CanRes(event, ressed)
	local guid = Grid2:GetGUIDByFullName(ressed)
	local unitid = Grid2:GetUnitidByGUID(guid)

	self:Debug("ResComm_ResStart ressed", ressed, "unitid", unitid, "guid", guid)
	unitSoulstone[unitid] = true
	if (Soulstone.enabled) then Soulstone:UpdateIndicators(unitid) end
end


function Grid2StatusRes:HealthChanged(units)
	for unitid in pairs(units) do
		if (not UnitIsDead(unitid)) then
			if (ResIncoming.enabled and unitResIncoming[unitid]) then
				unitResIncoming[unitid] = nil
				ResIncoming:UpdateIndicators(unitid)
			end
			if (Resurrected.enabled and unitResurrected[unitid]) then
				unitResurrected[unitid] = nil
				Resurrected:UpdateIndicators(unitid)
			end
			if (Soulstone.enabled and unitSoulstone[unitid]) then
				unitSoulstone[unitid] = nil
				Soulstone:UpdateIndicators(unitid)
			end
		end
	end
end




local prev_MakeDefaultSetup = Grid2.MakeDefaultSetup
function Grid2:MakeDefaultSetup(...)
	local setup, class = prev_MakeDefaultSetup(self, ...)

	if (not setup.res) then
		setup.res = true

		local setupIndicator = setup.status

		self:SetupIndicatorStatus(setupIndicator, "text-down", "resurrection-incoming", 99)
		self:SetupIndicatorStatus(setupIndicator, "text-down", "resurrected", 98)
		self:SetupIndicatorStatus(setupIndicator, "text-down", "soulstone", 97)

		self:SetupIndicatorStatus(setupIndicator, "text-down-color", "resurrection-incoming", 99)
		self:SetupIndicatorStatus(setupIndicator, "text-down-color", "resurrected", 98)
		self:SetupIndicatorStatus(setupIndicator, "text-down-color", "soulstone", 97)
	end

	return setup, class
end


-- Hook the loading of options so our associated lod options get loaded at the right time.
local prev_LoadOptions = Grid2.LoadOptions
function Grid2:LoadOptions(...)
	prev_LoadOptions(self, ...)
	if (Grid2StatusResOptions) then
		return
	end
	if (not IsAddOnLoaded("Grid2StatusResOptions")) then
		LoadAddOn("Grid2StatusResOptions")
	end
end
